import sys, os, re, signal, time, random, datetime

from ptrace import PtraceError
from ptrace.debugger import (PtraceDebugger, ProcessExit, ProcessSignal, NewProcessEvent, ProcessExecution)
from ptrace.syscall import SYSCALL_NAMES, SYSCALL_PROTOTYPES, FILENAME_ARGUMENTS, PtraceSyscall
from ptrace.func_call import FunctionCallOptions
from ptrace.syscall.socketcall import SOCKETCALL
from ptrace.compatibility import any
from ptrace.error import PTRACE_ERRORS, writeError
from ptrace.binding.func import PTRACE_O_TRACEFORK, PTRACE_O_TRACEVFORK
from ptrace.binding import ptrace_traceme, HAS_PTRACE_EVENTS

from optparse import OptionParser
from logging import (getLogger, debug, error, warning, info, StreamHandler, NOTSET, DEBUG, INFO, WARNING, ERROR)

from color import Color
from bad import BAD_SYSCALLS, BAD_OPEN_MODE, FILE_SYSCALLS
from emulprocess import EmulateProcess, EmulateProcessRootkit

class Emulate :
    def __init__(self, emul=True, pdf=False) :
        self.eprocess = {}
        self.filterfiles = None
        self.injectfiles = None
        self.emul = emul
        self.pdf = pdf
        
    def set_filterfiles(self, filter) :
        self.filterfiles = filter

    def set_injectfiles(self, inject) :
        self.injectfiles = inject
        
    def add_process(self, process) :
        if self.emul :
            self.eprocess[process.pid] = EmulateProcess(process, self.pdf)
        else :
            self.eprocess[process.pid] = EmulateProcessRootkit(process)

    def add_syscall(self, process, syscall, block) :
        self.eprocess[process.pid].add_syscall(syscall, block)
        if syscall.name in FILE_SYSCALLS :
            self.eprocess[process.pid].set_filterfiles(syscall, self.filterfiles)
            self.eprocess[process.pid].set_injectfiles(syscall, self.injectfiles)
        
    def emulate_syscall_enter(self, process, syscall) :
        return self.eprocess[process.pid].emulate_syscall_enter(syscall)
        
    def emulate_syscall_exit(self, process, syscall) :
        return self.eprocess[process.pid].emulate_syscall_exit(syscall)

    def get_block(self, process, syscall) :
        return self.eprocess[process.pid].get_block(syscall)

    def view_stats(self) :
        logger = getLogger()
        lv = logger.getEffectiveLevel()
        logger.setLevel(INFO)
        
        for i in self.eprocess :
            info("%s################## Stats %s%d%s ##################%s" % (Color.cyan, Color.yellow, i, Color.cyan, Color.normal))
            self.eprocess[i].view_stats()
            info("%s################## End Stats %s%d%s ##################%s" % (Color.cyan, Color.yellow, i, Color.cyan, Color.normal))

        logger.setLevel(lv)
        
class SyscallNode :
    def __init__(self, name) :
        self.name = name
        self.color = self.randcol()
        self.savcolor = self.color
        self.nb = 0
 
    def randcol(self):
        return (random.random(),random.random(),random.random())

class GraphSyscall :
    def __init__(self, process) :
        self.lprocess = {}
        self.lprocess[process.pid] = {}
            
    def add_process(self, process) :
        self.lprocess[process.pid] = {}

class FilterSyscalls :
    def __init__(self) :
        self.only = set()
        self.exclude = set()

    def add_expr(self, ctx, expr) :
        info("%sAdding filter %s%s" % (Color.yellow, expr, Color.normal))
        if (ctx == "only") :
            s = self.only
        elif (ctx == "exclude" ) :
            s = self.exclude
        else :
            return

        try :
            prog = re.compile(expr)
        except Exception, err :
            error("Invalid expression : %s" % err)
            return
        
        valid_names = SYSCALL_NAMES.values()
        for i in valid_names :
            if prog.search(i) :
                if not i in s :
                    s.add(i)
                    info("%sAdded %s%s" % (Color.blue, i, Color.normal))
            
    def del_expr(self, ctx, expr) :
        info("%sDeleting filter %s%s" % (Color.yellow, expr, Color.normal))
        if (ctx == "only") :
            s = self.only
        elif (ctx == "exclude" ) :
            s = self.exclude
        else :
            return

        try :
            prog = re.compile(expr)
        except Exception, err :
            error("Invalid expression : %s" % err)
            return
        
        l = []
        for i in s :
            if prog.search(i) :
                if i in s :
                    l.append(i)

        if l :
            for i in l :
                s.remove(i) 
                info("%sDeleted %s%s" % (Color.blue, i, Color.normal))

    def add_sockets(self) :
        info("%sAdding sockets%s" % (Color.purple, Color.normal))
        for syscall, prototype in SOCKETCALL.itervalues():
            self.only.add(syscall)
            info("%sAdded %s%s" % (Color.blue, syscall, Color.normal))
            
    def add_badsyscalls(self) :
        info("%sAdding bad syscalls%s" % (Color.purple, Color.normal))
        for i in BAD_SYSCALLS :
            self.add_expr("only", "^" + i + "$")
        self.add_sockets()

    def isPresent(self, ctx, name) :
        if (ctx == "only") :
            s = self.only
        elif (ctx == "exclude" ) :
            s = self.exclude
        else :
            return False

        if not s :
            return False
        
        if name in s :
            return True

        return False

    def isExist(self, ctx) :
        if (ctx == "only") :
            s = self.only
        elif (ctx == "exclude" ) :
            s = self.exclude
        else :
            return False

        if not s :
            return False

        return True
        
    def view(self, ctx) :
        info("%sViewing ctx %s%s" % (Color.purple, ctx, Color.normal))
        if (ctx == "only") :
            s = self.only
        elif (ctx == "exclude" ) :
            s = self.exclude
        else :
            return

        for i in s :
            info("%sSyscall %s%s" % (Color.blue, i, Color.normal))
        
    def reset(self, args=None) :
        info("%sReseting filter%s" % (Color.purple, Color.normal))
        if not args :
            self.only = set()
            self.exclude = set()
            info("%sReseted filter only%s" % (Color.blue, Color.normal))
            info("%sReseted filter exclude%s" % (Color.blue, Color.normal))
        elif args == "only" :
            self.only = set()
            info("%sReseted filter only%s" % (Color.blue, Color.normal))
        elif args == "exclude" :
            self.exclude = set()
            info("%sReseted filter exclude%s" % (Color.blue, Color.normal))

class FilterFiles :
    def __init__(self) :
        self.only = set()

    def add_expr(self, expr) :
        info("%sAdding filter %s%s" % (Color.yellow, expr, Color.normal))
        if expr not in self.only :
            self.only.add(expr)
            info("%sAdded %s%s" % (Color.blue, expr, Color.normal))

    def del_expr(self, expr) :
        info("%sDeleting filter %s%s" % (Color.yellow, expr, Color.normal))
        if expr in self.only :
            self.only.remove(expr)
            info("%sDeleted %s%s" % (Color.blue, expr, Color.normal))

    def isPresent(self, file) :
        for i in self.only :
            try :
                prog = re.compile(i)
                if prog.search(file) :
                    return True
            except Exception, err :
                error("Invalid expression : %s" % err)
                return False

        return False

    def view(self) :
        info("%sViewing filter files %s" % (Color.purple, Color.normal))

        for i in self.only :
            info("%sFilter %s%s" % (Color.blue, i, Color.normal))
            
    def reset(self, args=None) :
        info("%sReseting filter files%s" % (Color.purple, Color.normal))
        self.only = set()
        info("%sReseted filter%s" % (Color.blue, Color.normal))

class InjectFiles :
    def __init__(self) :
        self.only = set()

    def add_file(self, expr) :
        info("%sAdding file %s%s" % (Color.yellow, expr, Color.normal))
        if expr not in self.only :
            self.only.add(expr)
            info("%sAdded %s%s" % (Color.blue, expr, Color.normal))

    def del_file(self, expr) :
        info("%sDeleting file %s%s" % (Color.yellow, expr, Color.normal))
        if expr in self.only :
            self.only.remove(expr)
            info("%sDeleted %s%s" % (Color.blue, expr, Color.normal))

    def isPresent(self, fichier) :
        for i in self.only :
            name = i.name.replace('\x00', '')
            fichier = fichier[fichier.find("/")+1:]
            #print fichier
            if name == fichier :
                return i
            
        return None
    
    def view(self) :
        info("%sViewing files %s" % (Color.purple, Color.normal))

        for i in self.only :
            info("%sFile %s%s" % (Color.blue, i, Color.normal))
            
    def reset(self, args=None) :
        info("%sReseting files%s" % (Color.purple, Color.normal))
        self.only = set()
        info("%sReseted filtes%s" % (Color.blue, Color.normal))
        
class Box :
    def __init__(self, emul=True, viewstats=False) :
        self.program = None
        self.emul = None
        self.nb = 0
        
        self.is_gui = False
        self.d = None
        
        self.raw_socketcall = False
        self.askme = False
        self.askme_para = True

        self.block = 0

        self.viewstats = viewstats
        
        self.filter = FilterSyscalls()
        self.filterfiles = FilterFiles()
        self.injectfiles = InjectFiles()
        
        # Full emulation ?
        self.oemul = emul

        self.pdf = False

    def newProcess(self, event):
        process = event.process
        error("*** New process %s ***" % process.pid)
        
        self.emul.add_process(process)
        if self.is_gui :
            self.graphsyscall.add_process(process)

        self.prepareProcess(process)
        process.parent.syscall()

    def createChild(self, program):
        # Fork process
        pid = os.fork()
        if pid:
            return pid

        # Child code
        try:
            ptrace_traceme()
        except PtraceError, err:
            error("CHILD PTRACE ERROR! %s" % err)
            sys.exit(1)
            
        try:
            os.execv(program[0], program)
        except OSError, err:
            error("CHILD EXECVE ERROR! %s" % err)
            code = err.errno
        except Exception, err:
            error("CHILD EXECVE ERROR! %s" % err)
            code = 1
        exit(code)
        
    def prepareProcess(self, process):
        process.syscall()

    def processExecution(self, event):
        process = event.process
        error("*** Process %s execution ***" % process.pid)
        process.syscall()

    def walk(self) :
        is_attached = False

        try :
            self.debugger = PtraceDebugger()
            self.debugger.traceFork()
            self.debugger.traceExec()

            if self.program[0].isdigit() :
                pid = int(self.program[0])
                file = "/proc/" + self.program[0] + "/stat"
		try :
			fd = open(file, "r")
		except IOError, e :
			print "[-] IOError %s " % e
			return
		
		liste = fd.readlines()
		fd.close

                for j in liste :
                    l = j.split()
                    l[1] = l[1][1:]
                    l[1] = l[1][:-1]
                    self.program[0] = l[1]
                    
                is_attached = False
            else :
                pid = self.createChild(self.program)
                is_attached = True

            try:
                self.process = self.debugger.addProcess(pid, is_attached)
                self.emul.add_process(self.process)

                if self.is_gui :
                    self.graphsyscall = GraphSyscall(self.process)
            
            except (ProcessExit, PtraceError), err:
                error("ERROR: Process can no be attached! %s" % err)

            if not self.process:
                return

            self.syscall_options = FunctionCallOptions(
                write_types=False,
                write_argname=False,
                string_max_length=300,
                replace_socketcall=not self.raw_socketcall,
                write_address=False,
                max_array_count=20,
                )
            
            self.active_syscalls = {}

            self.syscallTrace(self.process)

        except ProcessExit, event:
            self.processExited(event)
        except PtraceError, err:
            error("ptrace() error: %s" % err)
        except KeyboardInterrupt:
            error("Interrupted! (CTRL+c)")
        except PTRACE_ERRORS, err:
            writeError(getLogger(), err, "Debugger error")

        if self.viewstats :
            self.emul.view_stats()
        self.debugger.quit()
