###############################################################################
# istari -- see http://www.esiea-recherche.eu/~desnos                         #
#                                                                             #
# The project istari is (C) 2009 : desnos at esiea.fr                         #
# This program is free software;                                              #
# you can redistribute it and/or modify it under the terms of the GNU         #
# General Public License as published by the Free Software Foundation;        #
# Version 2. This guarantees your right to use, modify, and                   #
# redistribute this software under certain conditions.                        #
#                                                                             #
# Source is provided to this software because we believe users have a         #
# right to know exactly what a program is going to do before they run         #
# it.                                                                         #
#                                                                             #
# This program is distributed in the hope that it will be                     #
# useful, but WITHOUT ANY WARRANTY; without even the implied                  #
# warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR                     #
# PURPOSE. See the GNU General Public License for more details (              #
# http://www.gnu.org/copyleft/gpl.html ).                                     #
#                                                                             #
###############################################################################

from ptrace.os_tools import RUNNING_LINUX, RUNNING_FREEBSD

if RUNNING_LINUX:
    from ptrace.syscall.linux_struct import (
        timeval, timespec, pollfd, rlimit, new_utsname, user_desc)
    from ptrace.syscall.linux_constants import SYSCALL_ARG_DICT, FD_SETSIZE
elif RUNNING_FREEBSD:
    from ptrace.syscall.freebsd_constants import SYSCALL_ARG_DICT
else:
    SYSCALL_ARG_DICT = {}
    
KNOWN_STRUCTS = []
if RUNNING_LINUX:
    KNOWN_STRUCTS.extend((timeval, timespec, pollfd, rlimit, new_utsname, user_desc))
KNOWN_STRUCTS = dict( (struct.__name__, struct) for struct in KNOWN_STRUCTS )

from ptrace.syscall.posix_arg import (
    formatMmapProt, formatAccessMode, formatOpenMode, formatCloneFlags)

from ptrace.signames import signalName

from log import LOGFILE
from logging import (log, getLogger, debug, error, warning, info, StreamHandler, DEBUG, INFO, WARNING, ERROR)

from bad import BAD_SYSCALLS, BAD_OPEN_MODE, BAD_MMAP_PROT
from stats import StatsSyscall
from color import Color
from ptrace.ctypes_tools import uint2int

from ctypes import (Structure, Union, sizeof, cast, POINTER,
    c_char, c_ubyte, c_ushort, c_int, c_uint, c_ulong, c_long, c_longlong, c_ulonglong, c_void_p)
from ptrace.ctypes_stdint import uint16_t, uint32_t, uint64_t

from linux_struct import dirent, stat64

import re

class EmulateSyscalls :
    def __init__(self, pid, pdf=False) :
        self.syscalls = {}
        self.stats = StatsSyscall(pid, pdf)
        self.open = {}
        
        self.lsyscalls = {}
        
    def bind(self, esyscall, name) :
        self.syscalls[esyscall.name].bind(esyscall)
        self.syscalls[esyscall.name].register(self.stats.get_stats(name))

        self.syscalls[esyscall.name].open = self.open
        
    def emulate_enter(self, esyscall) :
        self.syscalls[esyscall.name].emulate_enter()
        if esyscall.name not in self.lsyscalls :
            self.lsyscalls[esyscall.name] = 1
        else :
            self.lsyscalls[esyscall.name] += 1
        
    def emulate_exit(self, esyscall) :
        self.syscalls[esyscall.name].emulate_exit()
        
    def add_syscall(self, esyscall) :
        self.syscalls[esyscall.name] = esyscall

    def get_block(self, esyscall) :
        if self.syscalls[esyscall.name].block == True :
            return 1

        return 0

    def view_stats(self) :
        l = []
        for i in self.lsyscalls :
            l.append([i, self.lsyscalls[i]])

        l.sort(cmp=lambda x,y: cmp(x[1], y[1]))
        l.reverse()
        
        for i in l :
            info("%s%-30s ==> %s%d%s" % (Color.blue, i[0], Color.yellow, int(i[1]), Color.normal))
            
        self.stats.view()
        
class EmulateSyscall :
    def __init__(self, name, process, paranoia=True) :
        self.name = name
        self.process = process
        self.paranoia = paranoia
        self.musthijack = False
        self.block = False
        self.r = None
        self.filterfiles = None
        self.injectfiles = None
        self.open = {}
        
    def bind(self, syscall) :
        self.syscall = syscall
    
    def emulate_enter(self) :
        pass

    def emulate_exit(self) :
        pass
    
    def register(self, r) :
        self.r = r

    def hijack(self) :
        regs = self.process.getregs()
        regs.orig_eax = 300
        self.process.setregs(regs)
        self.block = True
        
    def set_musthijack(self) :
        if self.musthijack == True :
            self.hijack()
            
class SyscallBlock(EmulateSyscall) :
    def bind(self, syscall) :
        self.syscall = syscall
        self.block = True
    
    def emulate_enter(self) :
        info("ENTER BLOCK SYSCALL %s" % self.syscall.name)
        self.r.add(self.syscall.name, self.syscall.format(), not self.block)
        self.hijack()
        
    def emulate_exit(self) :
        regs = self.process.getregs()
        regs.eax = -1
        self.process.setregs(regs)

class SyscallOk(EmulateSyscall) :
    def bind(self, syscall) :
        self.syscall = syscall
        self.block = False
        
    def emulate_enter(self) :
        info("ENTER OK SYCALL %s" % self.syscall.name)
        self.r.add(self.syscall.name, self.syscall.format(), not self.block)
        pass
        
    def emulate_exit(self) :
        pass
        
class SyscallOpen(EmulateSyscall) :
    def emulate_enter(self) :
        self.set_musthijack()
        
        val = self.syscall.arguments[0].value
        path, truncated = self.process.readCString(val, 300)
        path = repr(path)
        if truncated:
            path += "..."

        mode = formatOpenMode(self.syscall.arguments[1])

        if self.paranoia == False :
            ok = False

            m = int(self.syscall.arguments[1].value)
            for i in BAD_OPEN_MODE :
                if m & i[0] :
                    info("%s Blocked syscall %s%s" % (Color.red, self.name, Color.normal))
                    self.hijack()
                    ok = True
                    break

            if ok == False :
                info("%s Not Blocked syscall %s%s" % (Color.blue, self.name, Color.normal))

        if self.r != None:
            self.r.add(path, mode, not self.block)
            
        return True

class SyscallCreat(SyscallOpen) :
    pass

class SyscallCreat(EmulateSyscall) :
    def emulate_enter(self) :
        self.set_musthijack()
        
        val = self.syscall.arguments[0].value
        path, truncated = self.process.readCString(val, 300)
        path = repr(path)
        if truncated:
            path += "..."

        mode = formatOpenMode(self.syscall.arguments[1])

        if self.paranoia == False :
            ok = False

            m = int(self.syscall.arguments[1].value)
            for i in BAD_OPEN_MODE :
                if m & i[0] :
                    info("%s Blocked syscall %s%s" % (Color.red, self.name, Color.normal))
                    self.hijack()
                    ok = True
                    break

            if ok == False :
                info("%s Not Blocked syscall %s%s" % (Color.blue, self.name, Color.normal))

        if self.r != None:
            self.r.add(path, mode, not self.block)
            
        return True
    
class SyscallUnlink(EmulateSyscall) :
    def emulate_enter(self) :
        self.set_musthijack()

        val = self.syscall.arguments[0].value
        path, truncated = self.process.readCString(val, 300)
        path = repr(path)
        if truncated:
            path += "..."
        
        if self.paranoia == False :
            info("%s Blocked syscall %s%s" % (Color.red, self.name, Color.normal))
            self.hijack()

        if self.r != None:
            self.r.add(path, not self.block)
            
        return True
    
class SyscallUname(EmulateSyscall) :
    def param(self, sysname, nodename, release, version, machine, domainname) :
        self.sysname = sysname
        self.nodename = nodename
        self.release = release
        self.version = version
        self.machine = machine
        self.domainname = domainname

    def emulate_enter(self) :
        self.set_musthijack()

        return True
    
    def emulate_exit(self) :
          for i in self.syscall.arguments :
#              struct = KNOWN_STRUCTS["new_utsname"]       
#              self.process.writeBytes(i.value + struct.sysname.offset, self.sysname)
#              self.process.writeBytes(i.value + struct.nodename.offset, self.nodename)
#              self.process.writeBytes(i.value + struct.release.offset, self.release)
#              self.process.writeBytes(i.value + struct.version.offset, self.version)
#              self.process.writeBytes(i.value + struct.machine.offset, self.machine)
#              self.process.writeBytes(i.value + struct.domainname.offset, self.domainname)

#              regs = self.process.getregs()
#              regs.eax = 0
#              self.process.setregs(regs)

              if self.r != None:
                  self.r.add("0x%x" % i.value, not self.block)
            
class SyscallPtrace(EmulateSyscall) :
    def emulate_enter(self) :
        self.set_musthijack()

        request = self.syscall.arguments[0].value
        pid = self.syscall.arguments[1].value
        addr = self.syscall.arguments[2].value
        data = self.syscall.arguments[3].value

        if self.paranoia == False :
            info("%s Blocked syscall %s%s" % (Color.red, self.name, Color.normal))
            self.hijack()
            
        if self.r != None:
            self.r.add("%d" % request, "%d" % pid, "0x%x" % addr, "0x%x" % data, not self.block)

        return True

class SyscallMmap(EmulateSyscall) :
    def bind(self, syscall) :
        self.syscall = syscall

    def emulate_enter(self) :
        self.set_musthijack()

        start = self.syscall.arguments[0].value
        length = self.syscall.arguments[1].value
        prot = formatMmapProt(self.syscall.arguments[2])
        flags = self.syscall.arguments[3].value
        fd = uint2int(self.syscall.arguments[4].value)
        offset = self.syscall.arguments[5].value
                
        if self.r != None:
            self.r.add("0x%x" % start, "%d" % length, "%s" % prot, "%d" % flags, "%d" % fd, "%d" % offset, not self.block)
        
class SyscallMunmap(EmulateSyscall) :
    def emulate_enter(self) :
        self.set_musthijack()

        start = self.syscall.arguments[0].value
        length = self.syscall.arguments[1].value
         
        if self.r != None:
            self.r.add("0x%x" % start, "%d" % length, not self.block)

class SyscallMprotect(EmulateSyscall) :
    def emulate_enter(self) :
        self.set_musthijack()

        start = self.syscall.arguments[0].value
        length = self.syscall.arguments[1].value
        prot = self.syscall.arguments[2].value

        if self.r != None:
            self.r.add("0x%x" % start, "%d" % length, "%d" % prot, not self.block)
            
class SyscallKill(EmulateSyscall) :
    def emulate_enter(self) :
        self.set_musthijack()

        pid = self.syscall.arguments[0].value
        sig = signalName(self.syscall.arguments[1].value)
        
        if self.paranoia == False :
            info("%s Blocked syscall %s%s" % (Color.red, self.name, Color.normal))
            self.block = True
            self.hijack()
            
        if self.r != None:
            self.r.add("%d" % pid, "%s" % sig, not self.block)

        return True

class SyscallTKill(EmulateSyscall) :
    def emulate_enter(self) :
        self.set_musthijack()

        tid = self.syscall.arguments[0].value
        sig =  signalName(self.syscall.arguments[1].value)
        
        if self.paranoia == False :
            info("%s Blocked syscall %s%s" % (Color.red, self.name, Color.normal))
            self.block = True
            self.hijack()
            
        if self.r != None:
            self.r.add("%d" % tid, "%s" % sig, not self.block)

        return True
    
class SyscallTGKill(EmulateSyscall) :
    def emulate_enter(self) :
        self.set_musthijack()

        tgid = self.syscall.arguments[0].value
        tid = self.syscall.arguments[1].value
        sig =  signalName(self.syscall.arguments[2].value)
        
        if self.paranoia == False :
            info("%s Blocked syscall %s%s" % (Color.red, self.name, Color.normal))
            self.block = True
            self.hijack()
            
        if self.r != None:
            self.r.add("%d" % tgid, "%d" % tid, "%s" % sig, not self.block)

        return True
    
class SyscallWrite(EmulateSyscall) :
    def emulate_enter(self) :
        self.set_musthijack()

        fd = self.syscall.arguments[0].value
        acontent = self.syscall.arguments[1].value
        length =  self.syscall.arguments[2].value

        if fd < 3 :
            truncated = (300 < length)
            size = min(length, 300)
            content = self.process.readBytes(acontent, size)
            content = repr(content)
            if truncated:
                content += "..."
        else :
            content = ("0x%x" % acontent)

        if self.r != None:
            self.r.add("%d" % fd, content, "%d" % length, not self.block)
            
        return True


class SyscallRead(EmulateSyscall) :
    def emulate_enter(self) :
        self.set_musthijack()

        fd = self.syscall.arguments[0].value
        content = self.syscall.arguments[1].value
        length =  self.syscall.arguments[2].value

        if self.r != None:
            self.r.add("%d" % fd, "0x%x" % content, "%d" % length, not self.block)
            
        return True

class SyscallClose(EmulateSyscall) :
    def emulate_enter(self) :
        self.set_musthijack()

        fd = self.syscall.arguments[0].value

        if self.r != None:
            self.r.add("%d" % fd, not self.block)
            
        return True

class SyscallFork(EmulateSyscall) :
    def emulate_enter(self) :
        self.set_musthijack()

        #print self.syscall.arguments[0].value
        
        if self.r != None:
            self.r.add(not self.block)
            
        return True

class SyscallVfork(EmulateSyscall) :
    def emulate_enter(self) :
        self.set_musthijack()

        if self.r != None:
            self.r.add(not self.block)
            
        return True
    
class SyscallBrk(EmulateSyscall) :
    def emulate_enter(self) :
        self.set_musthijack()

        end = self.syscall.arguments[0].value

        if self.r != None:
            self.r.add("0x%x" % end, not self.block)
            
        return True


    
class SyscallFstat(EmulateSyscall) :
    def emulate_enter(self) :
        self.set_musthijack()

        fd = self.syscall.arguments[0].value
        statbuf = self.syscall.arguments[1].value
        
        if self.r != None:
            self.r.add("%d" % fd, "0x%x" % statbuf, not self.block)
            
        return True

class SyscallLstat(EmulateSyscall) :
    def emulate_enter(self) :
        self.set_musthijack()

        val = self.syscall.arguments[0].value
        path, truncated = self.process.readCString(val, 300)
        path = repr(path)
        if truncated:
            path += "..."
            
        statbuf = self.syscall.arguments[1].value
        
        if self.r != None:
            self.r.add("%s" % path, "0x%x" % statbuf, not self.block)
            
        return True
    
class SyscallAccess(EmulateSyscall) :
    def emulate_enter(self) :
        self.set_musthijack()

        val = self.syscall.arguments[0].value
        path, truncated = self.process.readCString(val, 300)
        path = repr(path)
        if truncated:
            path += "..."

        mode = formatAccessMode(self.syscall.arguments[1])
        
        if self.r != None:
            self.r.add(path, mode, not self.block)
            
        return True

class SyscallRename(EmulateSyscall) :
    def emulate_enter(self) :
        self.set_musthijack()
        
        aold = self.syscall.arguments[0].value
        anew = self.syscall.arguments[1].value
        
        oldpath, truncated = self.process.readCString(aold, 300)
        oldpath = repr(oldpath)
        if truncated:
            oldpath += "..."

        newpath, truncated = self.process.readCString(anew, 300)
        newpath = repr(newpath)
        if truncated:
            newpath += "..."
            
        if self.paranoia == False :
            info("%s Blocked syscall %s%s" % (Color.red, self.name, Color.normal))
            self.hijack()

        if self.r != None:
            self.r.add(oldpath, newpath, not self.block)
            
        return True

class SyscallRenameat(EmulateSyscall) :
    def emulate_enter(self) :
        self.set_musthijack()

        olddirfd = self.syscall.arguments[0].value
        aold = self.syscall.arguments[1].value
        newdirfd = self.syscall.arguments[2].value
        anew = self.syscall.arguments[3].value
        
        oldpath, truncated = self.process.readCString(aold, 300)
        oldpath = repr(oldpath)
        if truncated:
            oldpath += "..."

        newpath, truncated = self.process.readCString(anew, 300)
        newpath = repr(newpath)
        if truncated:
            newpath += "..."
            
        if self.paranoia == False :
            info("%s Blocked syscall %s%s" % (Color.red, self.name, Color.normal))
            self.hijack()

        if self.r != None:
            self.r.add("%d" % olddirfd, oldpath, "%d" % newdirfd, newpath, not self.block)
            
        return True

class SyscallMkdir(EmulateSyscall) :
    def emulate_enter(self) :
        self.set_musthijack()
        
        val = self.syscall.arguments[0].value
        path, truncated = self.process.readCString(val, 300)
        path = repr(path)
        if truncated:
            path += "..."

        mode = self.syscall.arguments[1].value

        if self.paranoia == False :
            info("%s Blocked syscall %s%s" % (Color.red, self.name, Color.normal))
            self.hijack()
  
        if self.r != None:
            self.r.add(path, "%d" % mode, not self.block)
            
        return True

class SyscallMkdirat(EmulateSyscall) :
    def emulate_enter(self) :
        self.set_musthijack()

        dirfd = self.syscall.arguments[0].value
        val = self.syscall.arguments[1].value
        path, truncated = self.process.readCString(val, 300)
        path = repr(path)
        if truncated:
            path += "..."

        mode = self.syscall.arguments[2].value

        if self.paranoia == False :
            info("%s Blocked syscall %s%s" % (Color.red, self.name, Color.normal))
            self.hijack()
  
        if self.r != None:
            self.r.add("%d" % dirfd, path, "%d" % mode, not self.block)
            
        return True

class SyscallRmdir(EmulateSyscall) :
    def emulate_enter(self) :
        self.set_musthijack()
        
        val = self.syscall.arguments[0].value
        path, truncated = self.process.readCString(val, 300)
        path = repr(path)
        if truncated:
            path += "..."

        if self.paranoia == False :
            info("%s Blocked syscall %s%s" % (Color.red, self.name, Color.normal))
            self.hijack()
  
        if self.r != None:
            self.r.add(path, not self.block)
            
        return True

class SyscallModifyLdt(EmulateSyscall) :
    def emulate_enter(self) :
        self.set_musthijack()
        
        func = self.syscall.arguments[0].value
        ptr = self.syscall.arguments[1].value
        bytescount = self.syscall.arguments[2].value
        
        if self.paranoia == False :
            info("%s Blocked syscall %s%s" % (Color.red, self.name, Color.normal))
            self.hijack()
  
        if self.r != None:
            self.r.add("%d" % func, "0x%x" % ptr, "0x%x" % bytescount, not self.block)
            
        return True

class SyscallGetdents(EmulateSyscall) :
    def emulate_enter(self) :
        self.set_musthijack()

        fd = self.syscall.arguments[0].value
        dirp = self.syscall.arguments[1].value
        count = self.syscall.arguments[2].value
 
        if self.r != None:
            self.r.add("%d" % fd, "0x%x" % dirp, "%d" % count, not self.block)
            
        return True
    
class GFile :
    def __init__(self, dirent, name) :
        self.dirent = dirent
        self.name = name

        import ctypes
        self.d_ino = self.dirent.d_ino
        self.d_off = self.dirent.d_off
        self.d_reclen = self.dirent.d_reclen
        self.d_type = self.dirent.d_type
        self.d_name = self.dirent.d_name

    def view(self) :
        fileformat = "%-10d " % self.d_ino
        if self.d_type == 4 :
            fileformat += "D "
        elif self.d_type == 8 :
            fileformat += "F "
        else :
            fileformat += "U "

        fileformat += ("%-10d %-3d %s" % (self.d_off, self.d_reclen, self.name))
        info("%s" % fileformat)

class Directory :
    def __init__(self, dirp, process, regs, filter) :
        self.dirp = dirp
        self.process = process
        self.regs = regs
        self.filterfiles = filter
        self.files = []

    def add(self, dirent, name) :
        self.files.append(GFile(dirent, name))

    def hide(self, maxlen) :
        l = []
        for i in self.files :
            l.append(i)
            
        for i in l :
            if self.filterfiles.isPresent(i.name) :
                log(LOGFILE, "Hiding %s..." % i.name)
                self.files.remove(i)

        a = '\x00'
        a *= maxlen #self.regs.eax
        self.process.writeBytes(self.dirp, a)

        lenght = 0
        for j in self.files :
            #print j.d_reclen
            self.process.writeWord(self.dirp + lenght + dirent.d_ino.offset, j.d_ino)
            self.process.writeWord(self.dirp + lenght + dirent.d_off.offset, j.d_off)
            self.process.writeWord(self.dirp + lenght + dirent.d_reclen.offset, j.d_reclen)
            t = "%d" % j.d_type
            self.process.writeBytes(self.dirp + lenght + dirent.d_type.offset, t)
            self.process.writeBytes(self.dirp + lenght + dirent.d_name.offset, j.name)

            lenght += j.d_reclen
        
        info("Len %d" % lenght)
        self.regs.eax = lenght
        self.process.setregs(self.regs)

    def getfirst(self) :
        first = self.files[0]
        return first
    
    def getlast(self) :
        last = self.files[len(self.files) - 1]
        return last
        
    def view(self) :
        for i in self.files :
            i.view()

class SyscallGetdentsR(EmulateSyscall) :
    def emulate_exit(self) :        
        if self.filterfiles != None :            
            fd = self.syscall.arguments[0].value
            dirp = self.syscall.arguments[1].value
            count = self.syscall.arguments[2].value

            #print fd, dirp, count
            regs = self.process.getregs()
            
            #print regs.eax
            d = Directory(dirp, self.process, regs, self.filterfiles)

            nb = 0
            length = 0
            data = self.process.readStruct(dirp, dirent)
            name = dirp + dirent.d_name.offset
            
            while (length < regs.eax) :
                d.add(data, self.process.readCString(name, 300)[0])

                rlen = data.d_reclen
                            
                name += rlen

                data = self.process.readStruct(dirp + rlen + length, dirent)

                if data.d_ino != 0 :
                    nb += 1
                else :
                    regs.eax = length
                    
                length += rlen

            if regs.eax > 0 :
                if self.injectfiles != None :
                    # strip '
                    base = self.open[fd][1:-1]
                    baseinode = d.getfirst().d_ino                    
                    for i in self.injectfiles.only :
                        if i.base == base or ((base == "." or base == "./") and i.baseinode == baseinode) :
                            log(LOGFILE, "Injecting file %s in syscall %s ..." % (i.name, self.syscall.name))

                            newfile = dirent()
                            newfile.d_ino = i.inode
                            if len(i.name) % 2 :
                                i.name += "\x00"
                            
                            newfile.d_reclen = sizeof(dirent) + len(i.name)
                            mod = newfile.d_reclen % 4
                            if mod :
                                newfile.d_reclen += (newfile.d_reclen % 4)

                            newfile.d_off = d.getlast().d_off + newfile.d_reclen
                            newfile.d_type = i.type
                            d.add(newfile, i.name)
                            length += newfile.d_reclen

            d.view()
            d.hide(length)
            d.view()

        return True
        

class SyscallLstatR(EmulateSyscall) :
    def emulate_exit(self) :
        val = self.syscall.arguments[0].value
        path, truncated = self.process.readCString(val, 300)
        path = repr(path)
        if truncated:
            path += "..."
            
        statbuf = self.syscall.arguments[1].value

        #data = self.process.readStruct(statbuf, stat64)
        #print data.st_ino, data.st_mode
        
        if self.injectfiles != None :
            path = path[1:-1]
            #print path
            ret = self.injectfiles.isPresent(path)
            if ret != None :
                log(LOGFILE, "Injecting file %s in syscall %s ..." % (ret.name, self.syscall.name))

                #data = self.process.readStruct(statbuf, stat64)

                self.process.writeWord(statbuf + stat64.st_dev.offset, ret.dev)
                self.process.writeWord(statbuf + stat64.st_rdev.offset, ret.rdev)

                self.process.writeWord(statbuf + stat64.st_mode.offset, ret.mode)                
                self.process.writeWord(statbuf + stat64.st_nlink.offset, ret.nlink)
                
                self.process.writeWord(statbuf + stat64.st_uid.offset, ret.uid)
                self.process.writeWord(statbuf + stat64.st_gid.offset, ret.gid)

                self.process.writeWord(statbuf + stat64.st_size.offset, ret.size)
                self.process.writeWord(statbuf + stat64.st_blksize.offset, ret.blksize)

                self.process.writeWord(statbuf + stat64.st_blocks.offset, ret.blocks)

                self.process.writeWord(statbuf + stat64.st_atime.offset, ret.atime)
                self.process.writeWord(statbuf + stat64.st_atime_nsec.offset, ret.atimensec)
                
                self.process.writeWord(statbuf + stat64.st_mtime.offset, ret.mtime)
                self.process.writeWord(statbuf + stat64.st_mtime_nsec.offset, ret.mtimensec)

                self.process.writeWord(statbuf + stat64.st_ctime.offset, ret.ctime)
                self.process.writeWord(statbuf + stat64.st_ctime_nsec.offset, ret.ctimensec)
                
                self.process.writeWord(statbuf + stat64.st_ino.offset, ret.inode)
                            
                regs = self.process.getregs()
                regs.eax = 0
                self.process.setregs(regs)

        return True

class SyscallFstatR(EmulateSyscall) :
    def emulate_exit(self) :
        regs = self.process.getregs()

        fd = self.syscall.arguments[0].value
        statbuf = self.syscall.arguments[1].value
 
        if fd in self.open :
            if self.injectfiles != None :
                ret = self.injectfiles.isPresent(self.open[fd])
                if ret != None :
                    log(LOGFILE, "Injecting file %s in syscall %s ..." % (ret.name, self.syscall.name))

                    regs.eax = 0
                    self.process.setregs(regs)

        return True

class SyscallOpenR(EmulateSyscall) :
    def emulate_exit(self) :
        regs = self.process.getregs()
        val = self.syscall.arguments[0].value
        path, truncated = self.process.readCString(val, 300)
        path = repr(path)

        if truncated:
            path += "..."

        if uint2int(regs.eax) > 0 :
            self.open[regs.eax] = path
        else :
            if self.injectfiles != None :
                path = path[1:-1]
                ret = self.injectfiles.isPresent(path)
                if ret != None :
                    log(LOGFILE, "Injecting file %s in syscall %s ..." % (ret.name, self.syscall.name))
                    regs.eax = 9
                    self.process.setregs(regs)
                    self.open[regs.eax] = path

class SyscallCloseR(EmulateSyscall) :
    def emulate_exit(self) :
        regs = self.process.getregs()
        val = self.syscall.arguments[0].value
        if val in self.open :
            if self.injectfiles != None :
                ret = self.injectfiles.isPresent(self.open[val])
                if ret != None :
                    log(LOGFILE, "Injecting file %s in syscall %s ..." % (ret.name, self.syscall.name))
                    regs.eax = 0
                    self.process.setregs(regs)  

class SyscallReadR(EmulateSyscall) :
    def emulate_exit(self) :
        regs = self.process.getregs()
        val = self.syscall.arguments[0].value
        if val in self.open :
            if self.injectfiles != None :
                ret = self.injectfiles.isPresent(self.open[val])
                if ret != None :
                    log(LOGFILE, "Injecting file %s in syscall %s ..." % (ret.name, self.syscall.name))

                    if len(ret.content) < self.syscall.arguments[2].value :
                        maxi = len(ret.content)
                    else :
                        maxi = self.syscall.arguments[2].value

                    if ret.seek < len(ret.content) :
                        self.process.writeBytes(self.syscall.arguments[1].value, ret.content[ret.seek:maxi])

                        self.process.writeBytes(self.syscall.arguments[1].value, ret.content)
                        regs.eax = len(ret.content)
                        self.process.setregs(regs)  
                        ret.seek += maxi
                        
                    else :
                        regs.eax = 0
                        self.process.setregs(regs)  

        #fd = self.syscall.arguments[0].value
        #acontent = self.syscall.arguments[1].value
        #length =  self.syscall.arguments[2].value

        #regs = self.process.getregs()
        #content = self.process.readBytes(acontent, length)
        #nb = content.count("ptrace")
        #content = content.replace('ptrace', '')
            
        #self.nbwrite = (regs.eax - (nb * len("ptrace")))
        #self.process.writeBytes(self.syscall.arguments[1].value, content)

        #regs.eax = self.nbwrite
        #self.process.setregs(regs)

class SyscallWriteR(EmulateSyscall) :
    def emulate_enter(self) :
        pass

    def emulate_exit(self) :
        pass


class SyscallGetxattrR(EmulateSyscall) :
    def emulate_exit(self) :
        regs = self.process.getregs()
        if uint2int(regs.eax) == -2 :
            val = self.syscall.arguments[0].value
            path, truncated = self.process.readCString(val, 300)
            path = repr(path)
            if truncated:
                path += "..."
                
            if self.injectfiles != None :
                path = path[1:-1]
                ret = self.injectfiles.isPresent(path)
                if ret != None :
                    log(LOGFILE, "Injecting file %s in syscall %s ..." % (ret.name, self.syscall.name))
                    regs.eax = ret.getxattr[ret.posgetxattr]
                    ret.posgetxattr += 1
                    if ret.posgetxattr == 2 :
                        ret.posgetxattr = 0
                    self.process.setregs(regs)
